//
//  iMedia.h
//
//  Created by b4j on 9/17/14.
//  Copyright (c) 2014 Anywhere Software. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import "iCore.h"

@class B4IPage;
@class B4I;

/**
 *Lets the user choose a photo from the stored photos or take a photo with the camera.
 *Note that only portrait orientation is supported.
 */
//~version:1.23
//~shortname:Camera
//~Event:Complete (Success As Boolean, Image As Bitmap, VideoPath As String)
@interface B4ICamera : NSObject
//Checks whether camera is supported.
@property (nonatomic, readonly)BOOL IsSupported;
//Checks whether video camera is supported.
@property (nonatomic, readonly)BOOL IsVideoSupported;
//Gets or sets whether the front camera will be used by default.
@property (nonatomic) BOOL FrontCamera;
@property (nonatomic, readonly)int TYPE_IMAGE;
@property (nonatomic, readonly)int TYPE_MOVIE;
@property (nonatomic, readonly)int TYPE_ALL;
//Gets or sets whether the user can edit the selected image.
@property (nonatomic) BOOL AllowsEditing;

/**
* Initializes the object and sets the image picker parent.
*/
- (void)Initialize:(B4I *)bi :(NSString *)EventName :(B4IPage *)Page;

/**
* Shows the camera dialog. The camera will be set in video mode.
* Make sure to handle the Complete event
*/
- (void)TakeVideo;
/**
* Shows the camera dialog. The camera will be set in image mode.
* Make sure to handle the Complete event
*/
- (void)TakePicture;

/**
* Shows the photo library dialog. This dialog allows the user to select an image or movie from the device photo library.
*The Complete event will be raised.
* FromView - Relevant for iPad only. The dialog arrow will point to this view or BarButton.
*MediaType - One of the TYPE constants.
*/
- (void)SelectFromPhotoLibrary:(NSObject *)FromView :(int)MediaType;
/**
* Shows the saved photos dialog. This dialog allows the user to select an image or movie from the device camera roll album.
*The Complete event will be raised.
* FromView - Relevant for iPad only. The dialog arrow will point to this view or BarButton.
*MediaType - One of the TYPE constants.
*/
- (void)SelectFromSavedPhotos:(NSObject *)FromView :(int)MediaType;
@end

//~ShortName:VideoView
//~Event:Complete
//~Event:Ready
@interface B4IMoviePlayerWrapper:NSObject
//Returns the view itself. You should use this property to add the view.
@property (nonatomic, readonly)B4IViewWrapper *View;
//Returns the video duration in milliseconds. Returns 0 if the duration is not yet known.
@property (nonatomic, readonly)int Duration;
//Gets or sets the current position, measured in milliseconds.
@property (nonatomic)int Position;
//Set to true to show the video control panel. The default value is true.
@property (nonatomic)BOOL ShowControls;
//Initializes the object.
- (void)Initialize:(B4I *)bi :(NSString *)EventName;
//Loads the video from the given path.
- (void)LoadVideo:(NSString *)Dir :(NSString *)FileName;
//Streams the video from the given Url. The Ready event will fire when the buffer is ready.
- (void)LoadVideoUrl:(NSString *)Url;

//Starts or resumes playback.
- (void)Play;
//Pauses the playback.
- (void)Pause;
//Stops playback and sets the position to 0.
- (void)Stop;

@end
/**
* Provides low level access to the camera.
* This object allows you to embed the camera features inside your layout and provides more control over the camera settings.
* You can also work with the preview frames directly.
*/
//~shortname:LLCamera
//~dependson:AVFoundation.framework
//~event: PictureTaken (Data() As Byte)
//~event: Preview (Image As Bitmap)
@interface B4ILLCamera:NSObject<AVCaptureVideoDataOutputSampleBufferDelegate>
@property (nonatomic, readonly) int FLASH_OFF;
@property (nonatomic, readonly) int FLASH_ON;
@property (nonatomic, readonly) int FLASH_AUTO;
@property (nonatomic, readonly) int TORCH_OFF;
@property (nonatomic, readonly) int TORCH_ON;
@property (nonatomic, readonly) int TORCH_AUTO;
//Gets the maximum zoom factor.
@property (nonatomic, readonly) float ZOOM_MAX;
@property (nonatomic, readonly) NSString *PRESET_PHOTO;
@property (nonatomic, readonly) NSString *PRESET_HIGH;
@property (nonatomic, readonly) NSString *PRESET_MEDIUM;
@property (nonatomic, readonly) NSString *PRESET_LOW;
@property (nonatomic, readonly) NSString *PRESET_352x288;
@property (nonatomic, readonly) NSString *PRESET_640x480;
@property (nonatomic, readonly) NSString *PRESET_1280x720;
@property (nonatomic, readonly) NSString *PRESET_1920x1080;
//Gets or sets whether the preview image fills the panel (false) or the picture ratio is preserved.
@property (nonatomic)BOOL PreserveRatio;
/**
* Gets or sets the flash mode. The value should be one of the FLASH constants.
* You can only set this property inside a configuration block.
*/
@property (nonatomic)int FlashMode;
/**
* Gets or sets the torch mode. The value should be one of the TORCH constants.
* You can only set this property inside a configuration block.
*/
@property (nonatomic)int TorchMode;
/**
* Gets or sets the zoom factor. The value should be between 1 to ZOOM_MAX.
* You can only set this property inside a configuration block.
*/
@property (nonatomic)float Zoom;
//Returns true if the user has denied the app from using the camera.
@property (nonatomic, readonly)BOOL AuthorizationDenied;
//Gets or sets the preset mode. The value should be one of the PRESET constants.
@property (nonatomic)NSString *Preset;
//Starts a configuration block. Call CommitConfiguration to commit the changes.
- (void)BeginConfiguration;
//Commits the changes and ends the configuration block.
- (void)CommitConfiguration;

/**
* Initializes the camera.
*Panel - The preview panel.
*EventName - Sets the subs that will handle the events.
*FrontCamera - If true then the front camera will be used (if a front camera is available).
*/
- (void)Initialize:(B4I *)bi :(B4IPanelWrapper *)Panel :(NSString *)EventName :(BOOL)FrontCamera;

- (BOOL)IsInitialized;

/**
* Starts the data flow from the camera.
*/
- (void)StartPreview;

/**
* Stops the data flow.
*/
- (void)StopPreview;

/**
* This method should be called whenever the preview panel is resized or the orientation changes.
*/
- (void)Resize;


/**
* Takes a picture. The PictureTaken event will be raised. Note that this method does not stop the data flow (like in B4A).
*/
- (void)TakePicture;

/**
* This method should be called at the end of the Preview event with the bitmap passed as a parameter (only required if you are handling this event).
*/
- (void)ReleaseFrame:(B4IBitmap *)bmp;
@end
/**
* AudioRecorder allows you to record audio from the device microphone (or connected headset).
* The audio is saved to a file.
*/
//~shortname:AudioRecorder
//~objectwrapper:AVAudioRecorder*
@interface B4IAudioRecorder:B4IObjectWrapper
/**
* Initializes the recorder and sets the file that will hold the record data.
*Dir / FileName - The audio will be written to this file. Existing file will be deleted.
*SampleRate - Sample rate in Hz. Common values: 44100, 22050 and 11025.
*Mono - True for one channel false for two channels.
*Encoding - 8 bit or 16 bit encoding.
*MPEG4 - Whether to use MPEG4 encoding or linear PCM.
*/
- (void)Initialize:(NSString *)Dir :(NSString *)FileName :(int)SampleRate :(BOOL)Mono :(int)Encoding :(BOOL)MPEG4;

/**
* Similar to Initialize. Allows you to set the recorder settings. See Apple documentation for more information about the possible settings.
*/
- (void)Initialize2:(NSString *)Dir :(NSString *)FileName :(B4IMap *)Settings;
//Starts recording.
- (void)Record;
//Stops recording.
- (void)Stop;
@end
